
$(document).on('click', '#sendtonetsuite', function() {
    $.ajax({
        url: CreateSO_URL,
        type: 'get',
        data: {
            id: cartid
        },
        success: function(result, b, c) {
            console.log(result);
            if (result.status == 200) {
                $.ajax({
                    url: AddDeposit_URL,
                    type: 'get',
                    data: {
                        id: id
                    },
                    success: function(result, b, c) {
                        console.log(result);
                    }
                })

            }
        }
    })

})
$(document).on('click', '.link-netsuite-order', function() {
    $.ajax({
        url: LinkToNS_URL,
        type: 'get',
        data: {
            id: cartid
        },
        success: function(result, b, c) {
            $('#LGmodal .modal-body').html(result);
            $('#LGmodal').modal();
        }
    })
})